<?php

/**
 * @file
 * Form function for Ghost
 *
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Chris Skene chris at previousnext dot com dot au
 * @copyright Copyright(c) 2014 Previous Next Pty Ltd
 */

use Drupal\ghost\Form\FormInterface;
use Drupal\ghost\Exception\InvalidFormException;

/**
 * Custom form builder.
 *
 * @code
 * $form = drupal_get_form('ghost_form', '\Drupal\ghost\ExampleForm');
 * @code
 *
 * @param array $form
 *   (optional) The form.
 * @param array $form_state
 *   (optional) The form state.
 * @param string $form_handler
 *   The form handling class name.
 *
 * @throws Drupal\ghost\Exception\InvalidFormException
 * @return array
 *   Result of the buildForm operation.
 */
function ghost_form($form, &$form_state, $form_handler) {

  $form_class = ghost_get_form_handler($form_handler);

  $form_class->buildForm($form, $form_state);

  $form['ghost'] = array(
    '#type' => 'hidden',
    '#value' => $form_handler,
  );

  return $form;
}

/**
 * Form validation handler.
 *
 * @param array $form
 *   The form.
 * @param array $form_state
 *   The form state.
 *
 * @throws Drupal\ghost\Exception\InvalidFormException
 * @return mixed
 *   Result of the validateForm operation.
 */
function ghost_form_validate($form, &$form_state) {

  if (!isset($form['ghost'])) {

    throw new InvalidFormException('The submitted form used Ghost did not specify a Ghost form handler.');
  }

  $form_handler = ghost_get_form_handler($form['ghost']['#value']);

  return $form_handler->validateForm($form, $form_state);
}

/**
 * Form submit handler.
 *
 * @param array $form
 *   The form.
 * @param array $form_state
 *   The form state.
 *
 * @throws Drupal\ghost\Exception\InvalidFormException
 * @return mixed
 *   Result of the submitForm operation.
 */
function ghost_form_submit($form, &$form_state) {

  if (!isset($form['ghost'])) {

    throw new InvalidFormException('The submitted form used Ghost did not specify a Ghost form handler.');
  }

  $form_handler = ghost_get_form_handler($form['ghost']['#value']);

  return $form_handler->submitForm($form, $form_state);
}

/**
 * Construct a form handler.
 *
 * @param string $form_handler
 *   Name of the form handler.
 *
 * @throws Drupal\ghost\Exception\InvalidFormException
 * @return FormInterface
 *   A Form object.
 */
function ghost_get_form_handler($form_handler) {

  if (!class_exists($form_handler)) {
    throw new InvalidFormException('Could not find form handler class.');
  }

  $form_handler = new $form_handler();

  if (!$form_handler instanceof \Drupal\ghost\Form\FormInterface) {
    throw new InvalidFormException('The provided form handler class was not an instance of FormInterface.');
  }

  return $form_handler;
}
