<?php

/**
 * @file
 * Contains a FieldFormatterInterface
 */

namespace Drupal\ghost\Field;

use Drupal\image_grid\Grid\GridManager;
use Drupal\image_grid\Layout\LayoutManager;

/**
 * Interface FieldIFormatterInterface
 * @package Drupal\ghost\Field
 */
interface FieldFormatterInterface {

  /**
   * Version of hook_field_formatter_info().
   *
   * @see hook_field_formatter_info()
   *
   * Formatters handle the display of field values. Formatter hooks are
   * typically called by the Field Attach API field_attach_prepare_view() and
   * field_attach_view() functions.
   *
   * @return array
   *   An array describing the formatter types implemented by the module.
   *   The keys are formatter type names. To avoid name clashes, formatter type
   *   names should be prefixed with the name of the module that exposes them.
   *   The values are arrays describing the formatter type, with the following
   *   key/value pairs:
   *   - label: The human-readable name of the formatter type.
   *   - description: A short description for the formatter type.
   *   - field types: An array of field types the formatter supports.
   *   - settings: An array whose keys are the names of the settings available
   *     for the formatter type, and whose values are the default values for
   *     those settings.
   *
   * @see hook_field_formatter_info_alter()
   * @see hook_field_formatter_view()
   * @see hook_field_formatter_prepare_view()
   */
  public function info();

  /**
   * Specify the form elements for a formatter's settings.
   *
   * @see hook_field_formatter_settings_form()
   *
   * This hook is only invoked if hook_field_formatter_settings_summary()
   * returns a non-empty value.
   *
   * @param array $field
   *   The field structure being configured.
   * @param array $instance
   *   The instance structure being configured.
   * @param string $view_mode
   *   The view mode being configured.
   * @param array $form
   *   The (entire) configuration form array, which will usually
   *   have no use here.
   * @param array $form_state
   *   The form state of the (entire) configuration form.
   *
   * @return array
   *   The form elements for the formatter settings.
   */
  public function settingsForm($field, $instance, $view_mode, $form, &$form_state);

  /**
   * Return a short summary for the current formatter settings of an instance.
   *
   * @see hook_field_formatter_settings_summary()
   *
   * If an empty result is returned, the formatter is assumed to have no
   * configurable settings, and no UI will be provided to display a settings
   * form.
   *
   * @param array $field
   *   The field structure.
   * @param array $instance
   *   The instance structure.
   * @param string $view_mode
   *   The view mode for which a settings summary is requested.
   *
   * @return string
   *   A string containing a short summary of the formatter settings.
   */
  public function settingsSummary($field, $instance, $view_mode);

  /**
   * Build a renderable array for a field value.
   *
   * @see hook_field_formatter_view()
   *
   * @param string $entity_type
   *   The type of $entity.
   * @param object $entity
   *   The entity being displayed.
   * @param array $field
   *   The field structure.
   * @param array $instance
   *   The field instance.
   * @param string $langcode
   *   The language associated with $items.
   * @param array $items
   *   Array of values for this field.
   * @param array $display
   *   The display settings to use, as found in the 'display' entry of instance
   *   definitions. The array notably contains the following keys and values;
   *   - type: The name of the formatter to use.
   *   - settings: The array of formatter settings.
   *
   * @return array
   *   A renderable array for the $items, as an array of child elements keyed
   *   by numeric indexes starting from 0.
   */
  public function view($entity_type, $entity, $field, $instance, $langcode, $items, $display);
}
